/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include "ragrequesthandler.h"

#include "assistant/error.h"
#include "assistant/message.h"
#include "threadpool/async.h"
#include "utils/logger.h"

namespace {
const char *ragPrompt = R"(
    """# Role
        你是一个文档问答助手，可以结合给出的资料进行问题回复。
        # Context
        参考信息：<context>，我的问题或指令：<question>。
        # Rules
        请根据上述参考信息回答我的问题或回复我的指令。
        前面的参考信息可能有用，也可能没用，你需要从我给出的参考信息中选出与我的问题最相关的那些，来为你的回答提供依据。
        回答一定要忠于原文，简洁但不丢信息，不要胡乱编造。回答问题时尽量分条分点的回复。
        我的问题或指令是什么语种，你就用什么语种回复,你的回复："""
)";
}
assistant::ErrorCode RagRequestHandler::handleRequest(
    GDBusConnection *connection, const Json::Value &jsonMessage,
    assistant::session::DialogueSession &session) {
    LOGI("About to handle rag handler...");

    if (!useRag(session)) {
        return nextHandler_->handleRequest(connection, jsonMessage, session);
    }

    std::fprintf(stdout, "About to handle rag handler...\n");

    session.informationSources.clear();
    assistant::session::Message message = createMessage(session, jsonMessage);
    if (message.user.empty()) {
        return nextHandler_->handleRequest(connection, jsonMessage, session);
    }

    // 整理周报，生成待办等支持拖入文件的场景
    bool isChatNlpEngine = isOndeviceNlpEngineRequestWithRag(session);
    auto &engine = session.baseNlpEngine;
    auto &chatContext =
        isChatNlpEngine ? session.chatContext : session.ragChatContext;
    chatContext.messages.push_back(message);

    if (!engine) {
        std::fprintf(stderr, "Rag chat nlp engine is not set.\n");
        sendChatResult(
            connection, session.sessionId,
            assistant::message::creatreErrorMessageResult(
                (int)assistant::ErrorCode::NlpServerError, "模型加载失败"));
        return assistant::ErrorCode::NlpServerError;
    }

    const std::string chatMessage =
        assistant::session::chatContextToJsonString(chatContext);

    cpr::async(
        [&session, &engine, message, connection, chatMessage, &chatContext]() {
            std::fprintf(stdout, "Rag chat message: %s\n", chatMessage.c_str());
            ai_engine::EngineError engineError;
            if (!engine->chat(chatMessage, engineError)) {
                chatContext.messages.pop_back();
                assistant::ErrorCode errorCode =
                    assistant::Error::getErrorCodeFromNlpEngine(
                        engineError.getCode());
                sendChatResult(connection, session.sessionId,
                               assistant::message::creatreErrorMessageResult(
                                   (int)errorCode, engineError.getMessage()));
            }
        });
    return assistant::ErrorCode::Ok;
}

assistant::session::Message RagRequestHandler::createMessage(
    assistant::session::DialogueSession &session,
    const Json::Value &jsonMessage) {
    const std::string question =
        assistant::message::textContentFromMessage(jsonMessage);

    const std::string fileContent = getFileContent(session, question);
    if (fileContent.empty()) {
        std::fprintf(stdout, "RagResponseHandler::handleResponse: %s.\n",
                     "No similar file found1.");
    }

    assistant::session::Message message;
    if (!question.empty())
        message.user +=
            "context: " + fileContent + "\nquestion: " + question + "\n";
    else
        message.user += fileContent + "\n";
    std::fprintf(stdout, "Rag file content: %s.\n", fileContent.c_str());

    return message;
}

bool RagRequestHandler::isOndeviceNlpEngineRequestWithRag(
    const assistant::session::DialogueSession &session) {
    return isGetAllFileContentRequest(session);
}

bool RagRequestHandler::useRag(
    const assistant::session::DialogueSession &session) const {
    return !session.tmpKnownledgeBaseName.empty() ||
           session.useDefaultKnownledgeBase ||
           session.actorId == assistant::session::financeQaActorId;
}
bool RagRequestHandler::isGetAllFileContentRequest(
    const assistant::session::DialogueSession &session) const {
    return session.actorId ==
               assistant::session::weeklyReportGenerationActorId ||
           session.actorId == assistant::session::todoListGenerationActorId;
}

std::string RagRequestHandler::getFileContent(
    assistant::session::DialogueSession &session, const std::string &question) {
    if (isGetAllFileContentRequest(session)) {
        if (session.docFiles.empty()) {
            std::fprintf(stderr, "No doc files found.\n");
            return {};
        }
        return ragServerProxy_.getFileContent(session.tmpKnownledgeBaseName,
                                              session.docFiles.back());
    }

    const std::string knownledgeBaseName = getKnownledgeBaseName(session);
    auto fileContent = searchFileContent(question, knownledgeBaseName);

    std::string contents;
    for (auto &[filePath, content] : fileContent) {
        contents += content + "。";
        if (session.useDefaultKnownledgeBase) {
            const assistant::session::InformationSource informationSource =
                assistant::message::getInformationSourceFromUserGuidePath(
                    filePath);
            if (informationSource.id != -1) {
                session.informationSources.insert(informationSource);
            }
        }
    }
    return contents;
}

RagServerProxy::FileContentTupleVector RagRequestHandler::searchFileContent(
    const std::string &question, const std::string &knownledgeBaseName) {
    return ragServerProxy_.similaritySearch(knownledgeBaseName, question);
}

std::string RagRequestHandler::getKnownledgeBaseName(
    const assistant::session::DialogueSession &session) const {
    if (session.useDefaultKnownledgeBase) {
        return session.defaultKnownledgeBaseName;
    }

    if (session.actorId == assistant::session::financeQaActorId) {
        return session.financeKnownledgeBaseName;
    }

    return session.tmpKnownledgeBaseName;
}
