/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include "responsehandler.h"

#include <jsoncpp/json/value.h>

#include <cstdio>
#include <string>

#include "APIs/apiplatform.h"
#include "assistant/error.h"
#include "assistant/handlers/requesthandler.h"
#include "assistant/message.h"

namespace {

const char *cannotFindAppMessage = R"(
    {
        "message_type": "chat",
        "content": [
            {
                "type": "text",
                "text": {
                    "sentence_id": 1, "is_end": true, "result": "未找到该应用。"
                }
            }
        ],
        "error_code": 0,
        "error_message": "No error occurred."
    }
)";

const char *ragPrompt = R"(
    """# Role
        你是一个文档问答助手，可以结合给出的资料进行问题回复。
        # Context
        参考信息：<context>，我的问题或指令：<question>。
        # Rules
        请根据上述参考信息回答我的问题或回复我的指令。
        前面的参考信息可能有用，也可能没用，你需要从我给出的参考信息中选出与我的问题最相关的那些，来为你的回答提供依据。
        回答一定要忠于原文，简洁但不丢信息，不要胡乱编造。回答问题时尽量分条分点的回复。
        我的问题或指令是什么语种，你就用什么语种回复,你的回复："""
)";

}  // namespace

void SystemControlResponseHandler::handleResponse(
    const Json::Value &response, assistant::session::DialogueSession &session,
    GDBusConnection *connection) {
    const std::string intent = response[0]["intent_category"].asString();
    std::fprintf(stdout, "\x1b[1;31m");
    std::fprintf(stdout, "Intent: %s. Value: %s.\n", intent.c_str(),
                 response[0]["slots"].toStyledString().c_str());
    std::fprintf(stdout, "\x1b[0m");
    std::fprintf(stdout, "About to handle system control intent response...\n");
    if (intentManager_.isSystemControlIntent(intent)) {
        bool hasSlots = response[0]["slots"].size() > 0;
        const std::string value =
            hasSlots ? response[0]["slots"][0]["value"].asString() : "default";

        auto cmd = ApiPlatform::instance().createApi(intent, value);
        if (cmd && cmd->execute()) {
            const std::string resultMessage =
                assistant::message::createTextMessage(
                    "已为您完成操作。", assistant::model_name::assistantName);
            RequestHandler::sendChatResult(connection, session.sessionId,
                                           resultMessage);
        } else {
            std::fprintf(stderr,
                         "SystemControlResponseHandler::handleResponse, "
                         "no api for intent: %s\n",
                         intent.c_str());
            const std::string resultMessage =
                assistant::message::createTextMessage(
                    "执行失败。", assistant::model_name::assistantName);
            RequestHandler::sendChatResult(connection, session.sessionId,
                                           resultMessage);
        }
        handleComplete(session);

        return;
    }
    std::fprintf(stdout, "It is not system control intent response...\n");

    nextHandler_->handleResponse(response, session, connection);
}

VisionResponseHandler::VisionResponseHandler()
    /*: photoViewerPlugin_(std::make_unique<PhotoViewerPlugin>())*/{};

void VisionResponseHandler::handleResponse(
    const Json::Value &response, assistant::session::DialogueSession &session,
    GDBusConnection *connection) {
    std::fprintf(stdout, "About to handle vision intent response...%s\n",
                 response[0]["slots"][0]["value"].asString().c_str());
    const std::string intent = response[0]["intent_category"].asString();
    if (intentManager_.isVisionIntent(intent)) {
        if (response[0]["slots"][0]["value"].asString() == "文生图") {
            generateImage(session, connection);
        } else {
            processImage(response, session, connection);
        }
        handleComplete(session);
        return;
    }
    std::fprintf(stdout, "It is not vision intent response...\n");
    nextHandler_->handleResponse(response, session, connection);
}

void VisionResponseHandler::generateImage(
    assistant::session::DialogueSession &session, GDBusConnection *connection) {
    if (!session.visionEngine) {
        const std::string errorMessage =
            assistant::message::creatreErrorMessageResult(
                (int)assistant::ErrorCode::VisionServerError,
                "请配置Vision相关模型。");
        RequestHandler::sendChatResult(connection, session.sessionId,
                                       errorMessage);
        return;
    }

    const std::string prompt = session.intentChatContext.messages.back().user;
    ai_engine::EngineError engineError;
    if (!session.visionEngine->prompt2Image(prompt, engineError)) {
        assistant::ErrorCode errorCode =
            assistant::Error::getErrorCodeFromVisionEngine(
                engineError.getCode());
        const std::string errorMessage =
            assistant::message::creatreErrorMessageResult(
                (int)errorCode, engineError.getMessage());
        RequestHandler::sendChatResult(connection, session.sessionId,
                                       errorMessage);
    }
}

void VisionResponseHandler::processImage(
    const Json::Value &response, assistant::session::DialogueSession &session,
    GDBusConnection *connection) {
    if (session.sourceImageUrls.empty()) {
        fprintf(stderr, "No source image url\n");
        RequestHandler::sendChatResult(
            connection, session.sessionId,
            assistant::message::createTextMessage(
                "请先上传图片后再试。", assistant::model_name::assistantName));
        return;
    }
}

void RagResponseHandler::handleResponse(
    const Json::Value &response, assistant::session::DialogueSession &session,
    GDBusConnection *connection) {
    if (!useRag(session)) {
        nextHandler_->handleResponse(response, session, connection);
        return;
    }

    std::fprintf(stdout, "About to handle RAG response...\n");

    const std::string intent = response[0]["intent_category"].asString();

    if (intentManager_.isDocFileContentNeededIntent(intent)) {
        handleGetFileAllContentResponse(session);
        return;
    }

    handleResponseByCloudNlpEngineWithRag(response, session, connection);
}

bool RagResponseHandler::useRag(
    const assistant::session::DialogueSession &session) {
    return !session.tmpKnownledgeBaseName.empty() ||
           session.useDefaultKnownledgeBase;
}

void RagResponseHandler::handleGetFileAllContentResponse(
    assistant::session::DialogueSession &session) {
    std::fprintf(stdout, "About to get doc file content...\n");
    assistant::session::Message message{
        session.intentChatContext.messages.back().user};
    if (!session.docFiles.empty()) {
        const std::string fileContent = ragServerProxy_.getFileContent(
            session.tmpKnownledgeBaseName, session.docFiles.back());
        message.user += "。文档中的具体内容是：" + fileContent;
    } else {
        std::fprintf(stdout, "RagResponseHandler::handleResponse: %s.\n",
                     "No doc file found.");
    }

    // 不保存文档文档历史对话
    session.intentChatContext.messages.erase(
        session.intentChatContext.messages.end() - 1);

    assistant::session::ChatContext chatContext;
    chatContext.messages.push_back(message);
    const std::string chatMessage =
        assistant::session::chatContextToJsonString(chatContext);

    ai_engine::EngineError error;
    session.ondeviceNlpEngine->chat(chatMessage, error);
    handleComplete(session);
}

void RagResponseHandler::handleResponseByCloudNlpEngineWithRag(
    const Json::Value &response, assistant::session::DialogueSession &session,
    GDBusConnection *connection) {
    if (!session.cloudNlpEngine) {
        const std::string errorMessage =
            assistant::message::creatreErrorMessageResult(
                (int)assistant::ErrorCode::NlpServerError,
                "云端nlp引擎未设置，请检查是否配置云端nlp。");
        RequestHandler::sendChatResult(connection, session.sessionId,
                                       errorMessage);
        handleComplete(session);
        return;
    }

    const std::string question = session.intentChatContext.messages.back().user;
    const std::string knownledgeBaseName =
        session.useDefaultKnownledgeBase ? session.defaultKnownledgeBaseName
                                         : session.tmpKnownledgeBaseName;
    auto fileContent = searchFileContent(question, knownledgeBaseName);
    if (fileContent.empty()) {
        std::fprintf(stdout, "RagResponseHandler::handleResponse: %s.\n",
                     "No similar file found.");
        nextHandler_->handleResponse(response, session, connection);
        return;
    }

    handleRagSearchContentResponseByCloudNlpEngine(session, question,
                                                   fileContent);
}

RagServerProxy::FileContentTupleVector RagResponseHandler::searchFileContent(
    const std::string &question, const std::string &knownledgeBaseName) {
    return ragServerProxy_.similaritySearch(knownledgeBaseName, question);
}

void RagResponseHandler::handleRagSearchContentResponseByCloudNlpEngine(
    assistant::session::DialogueSession &session, const std::string &question,
    const RagServerProxy::FileContentTupleVector &fileContent) {
    session.informationSources.clear();

    std::string contents;
    for (auto &[filePath, content] : fileContent) {
        contents += "。" + content;
        if (session.useDefaultKnownledgeBase) {
            const assistant::session::InformationSource informationSource =
                assistant::message::getInformationSourceFromUserGuidePath(
                    filePath);
            if (informationSource.id != -1) {
                session.informationSources.insert(informationSource);
            }
        }
    }

    std::string prompt{ragPrompt};
    prompt.replace(prompt.find("<context>"), 9, contents);
    prompt.replace(prompt.find("<question>"), 10, question);

    std::fprintf(stdout, "\x1b[1;31m");
    std::fprintf(stdout, "RagRequestHandler::handleRequest: %s.\n",
                 prompt.c_str());
    std::fprintf(stdout, "\x1b[0m");

    ai_engine::EngineError error;
    if (!session.cloudNlpEngine->chat(prompt, error)) {
        std::fprintf(stderr, "RagRequestHandler::handleRequest: %s.\n",
                     error.getMessage().c_str());
    }
    handleComplete(session);
}

void BaichuanOnDeviceResponseHandler::handleResponse(
    const Json::Value &response, assistant::session::DialogueSession &session,
    GDBusConnection *connection) {
    std::fprintf(stdout, "About to handle ondevice intent response...\n");
    const std::string intent = response[0]["intent_category"].asString();

    std::string userQuestion = session.intentChatContext.messages.back().user;
    if (intentManager_.isOnDeviceNlpIntent(intent)) {
        handleOnDeviceNlpIntent(intent, session);
        return;
    }

    std::fprintf(stdout, "It is not ondevice intent...\n");
    nextHandler_->handleResponse(response, session, connection);
}

void BaichuanOnDeviceResponseHandler::handleOnDeviceNlpIntent(
    const std::string &intent, assistant::session::DialogueSession &session) {
    assistant::session::Message message{
        session.intentChatContext.messages.back().user};

    if (intentManager_.isDocFileContentNeededIntent(intent)) {
        std::fprintf(stdout, "About to get doc file content...\n");
        if (!session.docFiles.empty()) {
            const std::string fileContent = ragServerProxy_.getFileContent(
                session.tmpKnownledgeBaseName, session.docFiles.back());
            message.user += "。文档中的具体内容是：" + fileContent;
        }
    }

    session.chatContext.messages.push_back(message);
    const std::string chatMessage =
        assistant::session::chatContextToJsonString(session.chatContext);
    ai_engine::EngineError error;
    session.ondeviceNlpEngine->chat(chatMessage, error);
    handleComplete(session);
}

void NlpResponseHandler::handleResponse(
    const Json::Value &response, assistant::session::DialogueSession &session,
    GDBusConnection *connection) {
    std::fprintf(stdout, "About to handle cloud nlp intent response...\n");
    const std::string intent = response[0]["intent_category"].asString();

    if (intentManager_.isCloudNlpIntent(intent)) {
        if (!session.cloudNlpEngine) {
            const std::string errorMessage =
                assistant::message::creatreErrorMessageResult(
                    (int)assistant::ErrorCode::NlpServerError,
                    "云端nlp引擎未设置，请检查是否配置云端nlp。");
            RequestHandler::sendChatResult(connection, session.sessionId,
                                           errorMessage);
            std::fprintf(stderr, "NlpResponseHandler::handleResponse: %s.\n",
                         "Cloud nlp engine is not set.");
            handleComplete(session);
            return;
        }
        assistant::session::Message message{
            session.intentChatContext.messages.back().user};
        session.chatContext.messages.push_back(message);

        // 当前只有助手使用，而且是一问一答的形式，暂时不用考虑线程安全
        cpr::async([this, &session, message]() {
            ai_engine::EngineError error;
            session.cloudNlpEngine->chat(message.user, error);
            handleComplete(session);
        });

        return;
    }
    std::fprintf(stdout, "\x1b[1;31m");
    std::fprintf(stdout, "NlpResponseHandler::Unkwon intent: %s\n",
                 intent.c_str());
    std::fprintf(stdout, "\x1b[0m");

    if (!session.ondeviceNlpEngine) {
        std::fprintf(stderr, "NlpResponseHandler::handleResponse: %s.\n",
                     "On device nlp engine is not set.");
        handleComplete(session);
        return;
    }

    assistant::session::Message message{
        session.intentChatContext.messages.back().user};
    session.chatContext.messages.push_back(message);
    const std::string chatMessage =
        assistant::session::chatContextToJsonString(session.chatContext);
    ai_engine::EngineError error;
    session.ondeviceNlpEngine->chat(chatMessage, error);
    handleComplete(session);
}
