/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include "ragserverproxy.h"

#include <knowledgebaseclient.h>

#include <cstdio>
#include <string>

#include "assistant/message.h"
#include "utils/logger.h"
#include "utils/util.h"

namespace {
constexpr double similarityThreshold = 0.60;
constexpr unsigned int topK = 4;
}  // namespace

RagServerProxy &RagServerProxy::instance() {
    static RagServerProxy instance;
    return instance;
}

bool RagServerProxy::isValid() const {
    return KnowledgebaseClient::getInstance()->isSupported();
}
RagServerProxy::RagServerProxy() {
#if 0
    if (!init()) {
        if (!reInit()) {
            std::fprintf(stderr, "Init knowledge base processor failed.\n");
            return;
        }
    }
#endif
}

std::string RagServerProxy::createKnowledgeBase() {
    // return "temp-knowledgebase-name";

    std::string input = R"(
        {
            "knowledgebase_name" :  "default"
        }
    )";
    std::string knowledgebaseName;
    auto errorInfo = KnowledgebaseClient::getInstance()->createKnowledgebase(
        input, knowledgebaseName);

    if (errorInfo.errorCode != 0) {
        LOGE("Create knowledge base failed: {}.", errorInfo.errorMessage);
        return {};
    }

    return knowledgebaseName;
}

void RagServerProxy::deleteKnowledgeBase(const std::string &knowledgeBaseName) {
    auto errorInfo = KnowledgebaseClient::getInstance()->deleteKnowledgebase(
        knowledgeBaseName);

    if (errorInfo.errorCode != 0) {
        LOGE("Delete knowledge base failed: {}.", errorInfo.errorMessage);
    }
}

RagErrorInfo RagServerProxy::addFiles(const std::string &knowledgeBaseName,
                                      const std::vector<std::string> &files) {
    // {
    //     "filepath": "/home/user/test.txt",
    //     "fileformat": "txt",
    // }
    LOGI("Add files to knowledge base: {}.", knowledgeBaseName);
    Json::Value fileInfos;
    for (const auto &file : files) {
        std::string fileFormat = file.substr(file.find_last_of(".") + 1);
        Json::Value fileInfo;
        fileInfo["filepath"] = file;
        fileInfo["fileformat"] = fileFormat;
        fileInfo["knowledgebase_name"] = knowledgeBaseName;
        fileInfo["chunk_size"] = 100;
        fileInfos.append(fileInfo);
    }

    LOGI(fileInfos.toStyledString().c_str());
    auto errorInfo = KnowledgebaseClient::getInstance()->addTextFiles(
        fileInfos.toStyledString());

    if (errorInfo.errorCode != 0) {
        LOGE("Add files to knowledge base failed: {}.", errorInfo.errorCode,
             errorInfo.errorMessage);
    }
    return {errorInfo.errorCode, errorInfo.errorMessage};
}

RagServerProxy::FileContentTupleVector RagServerProxy::similaritySearch(
    const std::string &knowledgeBaseName, const std::string &searchConditions) {
    std::string searchResultJsonStr;
    Json::Value searchInput;
    searchInput["knowledgebase_name"] = knowledgeBaseName;
    searchInput["search_condition"] = searchConditions;
    auto errorInfo = KnowledgebaseClient::getInstance()->similaritySearch(
        searchInput.toStyledString(), searchResultJsonStr);

    if (errorInfo.errorCode != 0) {
        LOGE("Similarity search failed: {}", errorInfo.errorMessage);
        return {};
    }

    LOGD("Similarity search result: {}.", searchResultJsonStr);
    Json::Value searchResultJson = util::stringToJson(searchResultJsonStr);

    auto ret = handleSearchResult(searchResultJson);
    LOGI("Similarity search result : {}", searchResultJson.toStyledString());
    return ret;
}

std::string RagServerProxy::getFileContent(const std::string &knowledgeBaseName,
                                           const std::string &filePath) {
    if (filePath.empty()) {
        return {};
    }

    Json::Value fileInfos;
    Json::Value fileInfo;
    const std::string fileFormat =
        filePath.substr(filePath.find_last_of(".") + 1);
    fileInfo["filepath"] = filePath;
    fileInfo["fileformat"] = fileFormat;
    fileInfo["knowledgebase_name"] = knowledgeBaseName;
    fileInfos.append(fileInfo);

    std::string jsonStrResult;
    auto errorInfo = KnowledgebaseClient::getInstance()->getTextFileContents(
        fileInfos.toStyledString(), jsonStrResult);
    if (errorInfo.errorCode != 0) {
        LOGE("Get file content failed: {}", errorInfo.errorMessage);
        return {};
    }

    LOGI("Get file content result: {}.", jsonStrResult);

    Json::Value result = util::stringToJson(jsonStrResult);
    return result.size() > 0 ? result[0]["document"].asString() : "";
}

RagServerProxy::FileContentTupleVector RagServerProxy::handleSearchResult(
    const Json::Value &jsonResult) {
    FileContentTupleVector result;
    LOGI("Similarity search result size: {}.", jsonResult.size());
    for (unsigned int i = 0; i < std::min(topK, jsonResult.size()); ++i) {
        if (jsonResult[i]["similarity"].asDouble() > similarityThreshold) {
            const std::string searchPath = jsonResult[i]["filepath"].asString();
            const std::string searchResultString =
                jsonResult[i]["document"].asString();
            LOGD("{} {}", searchPath, searchResultString);
            result.push_back(std::make_tuple(searchPath, searchResultString));
        }
        LOGI("Similarity search file path: {} similarity {}.",
             jsonResult[i]["filepath"].asString(),
             jsonResult[i]["similarity"].asDouble());
    }
    return result;
}
