/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include "util.h"

#include <glib.h>
#include <openssl/bio.h>
#include <openssl/buffer.h>
#include <openssl/evp.h>

#include <cstring>
#include <fstream>
#include <memory>
#include <sstream>

#include "logger.h"

namespace util {

std::string jsonToString(const Json::Value& value) {
#ifdef RUN_IN_DEBUG
    return value.toStyledString();
#else
    Json::FastWriter writer;
    return writer.write(value);
#endif
}

Json::Value stringToJson(const std::string& string) {
    Json::Value value;
    if (Json::Reader reader; reader.parse(string, value)) {
        return value;
    } else {
        return Json::objectValue;
    }
}

std::string currentDate(const std::string& format) {
    setlocale(LC_TIME, "C");
    time_t rawTime = time(nullptr);
    tm* gmt = localtime(&rawTime);
    char tstr[30];
    strftime(tstr, sizeof(tstr), format.c_str(), gmt);
    return tstr;
}

std::vector<uint8_t> base64Decode(const std::string& input) {
    gsize decodedLength{};
    guchar* decodedData = g_base64_decode(input.c_str(), &decodedLength);
    std::vector<uint8_t> result(decodedData, decodedData + decodedLength);
    g_free(decodedData);
    return result;
}

std::string base64Encode(const char* data, int length) {
    BIO *bio, *b64;
    BUF_MEM* bufferPtr;
    b64 = BIO_new(BIO_f_base64());
    BIO_set_flags(b64, BIO_FLAGS_BASE64_NO_NL);

    bio = BIO_new(BIO_s_mem());
    bio = BIO_push(b64, bio);
    BIO_write(bio, data, length);
    BIO_flush(bio);
    BIO_get_mem_ptr(bio, &bufferPtr);
    std::string result(bufferPtr->data, bufferPtr->length);

    BIO_free_all(bio);

    return result;
}

std::string base64Encode(const std::string& input) {
    return base64Encode(input.data(), input.size());
}

void removeSequenceFromString(std::string& str, const std::string& sequence) {
    size_t pos = 0;
    while ((pos = str.find(sequence, pos)) != std::string::npos) {
        str.erase(pos, sequence.length());
    }
}

void removeZeroWidthChar(std::string& str) {
    static const char* zeroWidthChars[] = {
        "\xE2\x80\x8B",  // Zero width space
        "\xE2\x80\x8C",  // Zero width non-joiner
        "\xE2\x80\x8D",  // Zero width joiner
        "\xE2\x80\x8E",  // Left to right mark
        "\xE2\x80\x8F",  // Right to left mark
    };
    for (const char* zeroWidthChar : zeroWidthChars) {
        removeSequenceFromString(str, zeroWidthChar);
    }
}

bool stringContains(const std::string& str, const std::string& sub) {
    if (str.length() < sub.length()) {
        return false;
    };
    return str.find(sub) != std::string::npos;
}

const std::string readFile(const std::string& fileName) {
    std::ifstream file(fileName);
    if (!file.is_open()) {
        return "";
    }
    return std::string(std::istreambuf_iterator<char>(file),
                       std::istreambuf_iterator<char>());
}

}  // namespace util
